//
// This file is part of AlphaCam II.
//
// AlphaCam II is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AlphaCam II is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with AlphaCam II.  If not, see <http://www.gnu.org/licenses/>.
//
//
//  NLColorPickerController.m
//  AlphaCam II
//
//  Created by Nicholas Levin on 5/26/11.
//  Copyright 2011 Studio Andromeda. All rights reserved.
//

#import "NLColorPickerController.h"

#import "NLCoreDataController.h"
#import "NLColorsEditorController.h"
#import "NLMOPalette.h"
#import "NLMOColor.h"

@implementation NLColorPickerController

@synthesize redField, greenField, blueField, currentPalette, currentColor, colorsEditorController, coreDataController;

- (id)initWithCoreDataController:(NLCoreDataController *)aCoreDataController palette:(NLMOPalette *)aPalette color:(NLMOColor *)aColor colorsEditorController:(NLColorsEditorController *)aColorsEditorController
{
    self = [super init];
    if (self) {
        // Custom initialization
        
        coreDataController = aCoreDataController;
        colorsEditorController = aColorsEditorController;
        currentPalette = aPalette;
        currentColor = aColor;
    }
    return self;
}


- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (IBAction)save:(id)sender
{
    if (coreDataController != nil) {
        if (currentColor != nil) {
            NSNumberFormatter * formatter = [[NSNumberFormatter alloc] init];
            [formatter setNumberStyle:NSNumberFormatterRoundFloor];
            
            currentColor.red = [formatter numberFromString:redField.text];
            currentColor.green = [formatter numberFromString:greenField.text];
            currentColor.blue = [formatter numberFromString:blueField.text];

            
            [coreDataController saveContext];
        } else {
            printf("ERROR: currentColor was not initialized in NLColorPickerController class.\n");
        }
        [colorsEditorController.tableView reloadData];
    }
    [self dismissModalViewControllerAnimated:YES];
}

- (IBAction)cancel:(id)sender
{
    [self dismissModalViewControllerAnimated:YES];
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
    if (currentColor != nil) {
        redField.text = [currentColor.red stringValue];
        greenField.text = [currentColor.green stringValue];
        blueField.text = [currentColor.blue stringValue];
    }
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
