//
// This file is part of AlphaCam II.
//
// AlphaCam II is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AlphaCam II is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with AlphaCam II.  If not, see <http://www.gnu.org/licenses/>.
//
//
//  NLColorsEditorController.m
//  AlphaCam II
//
//  Created by Nicholas Levin on 5/26/11.
//  Copyright 2011 Studio Andromeda. All rights reserved.
//

#import "NLColorsEditorController.h"

#import "NLColorPickerController.h"
#import "NLMOPalette.h"
#import "NLMOColor.h"

@implementation NLColorsEditorController

@synthesize coreDataController, currentPalette;

- (id)initWithStyle:(UITableViewStyle)style coreDataController:(NLCoreDataController*)CDController andPalette:(NLMOPalette*)palette
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
        
        self.coreDataController = CDController;
        self.currentPalette = palette;
    }
    return self;
}


- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (NSArray *)sortColors
{
    NSSortDescriptor *sortRedDescriptor = [[NSSortDescriptor alloc] initWithKey:@"red" ascending:YES];
    
    NSArray *sortDescriptors = [NSArray arrayWithObjects:sortRedDescriptor, nil];
    return [[[currentPalette color] allObjects] sortedArrayUsingDescriptors:sortDescriptors];
}

#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [currentPalette.color count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"OldComputerColorCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    }
    
    // Configure the cell...
    
    NLMOColor *color = [[self sortColors] objectAtIndex:indexPath.row];
    cell.textLabel.text = [NSString stringWithFormat:@"R:%i G:%i B:%i", [[color red] intValue], [[color green] intValue],[[color blue] intValue]];
        
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Navigation logic may go here. Create and push another view controller.
    
    NLMOColor *color = [[self sortColors] objectAtIndex:indexPath.row];
    NLColorPickerController *colorPickerController = [[NLColorPickerController alloc] initWithCoreDataController:coreDataController palette:currentPalette color:color colorsEditorController:self];

    [self presentModalViewController:colorPickerController animated:YES];
}

@end
