//
// This file is part of AlphaCam II.
//
// AlphaCam II is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AlphaCam II is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with AlphaCam II.  If not, see <http://www.gnu.org/licenses/>.
//
//
//  NLGLCameraVideoRecorder.m
//  AlphaCam II
//
//  Created by Nicholas Levin on 5/6/11.
//  Copyright 2011 Studio Andromeda. All rights reserved.
//

#import "NLGLCameraVideoRecorder.h"

#define OUTPUT_FILE_NAME @"sample.mov"  
// TODO: Make file names different for each movie recorded, so we don't overwrite previous movies.

#define FRAME_WIDTH 320
#define FRAME_HEIGHT 480

@implementation NLGLCameraVideoRecorder

@synthesize isRecording;

- (id)init {
    if ((self = [super init])) {
        
        if ([[UIScreen mainScreen] scale] == 2.0)
            //frameScale = 2.0;
            frameScale = 1.2;
        else
            frameScale = 1.0;
        
        isRecording = NO;
    }
    
    return self;
}

- (void)dealloc {
    
    assetWriter = nil;
	assetWriterInput = nil;
    assetWriterPixelBufferAdaptor = nil;
    free(rawImageData); rawImageData = nil;
    
    // cleanup
}

-(NSString*) pathToDocumentsDirectory {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	return documentsDirectory;
}

-(CVPixelBufferRef) retrievePooledPixelBuffer {
    
    CVPixelBufferRef pixelBuffer = NULL;    
    CVReturn cvErr = CVPixelBufferPoolCreatePixelBuffer(nil, [assetWriterPixelBufferAdaptor pixelBufferPool], &pixelBuffer);
    CVPixelBufferLockBaseAddress(pixelBuffer, 0);
    
    if (cvErr != kCVReturnSuccess) {
        
        printf("failed to create pixel buffer\n");
        
        CVPixelBufferUnlockBaseAddress(pixelBuffer, 0);
        CVBufferRelease(pixelBuffer);
    
        return nil; // if nil, then we should either crash the program safely or stop recording
    }
    
    //CVBufferRetain(pixelBuffer); // be sure to unlock the base address after we write the sample!
    return pixelBuffer;
}

-(void) writeSampleAtTime:(CMTime) presentationTime {
	if (assetWriterInput.readyForMoreMediaData) {
		
        // start the session if we're at the first frame
        if (isFirstFrame) {
            [assetWriter startSessionAtSourceTime: presentationTime];
            isFirstFrame = NO;
        }
                
        glReadPixels(0, 0, scaledWidth, scaledHeight, GL_BGRA_EXT, GL_UNSIGNED_BYTE, rawImageData);
        
        CVPixelBufferRef pixelBuffer = [self retrievePooledPixelBuffer];

        
        unsigned char* baseAddress = CVPixelBufferGetBaseAddress(pixelBuffer);
        unsigned rowbytes = CVPixelBufferGetBytesPerRow(pixelBuffer);
        
        unsigned char* src;
        unsigned char* dst;
        
        for(unsigned int i = 0; i < scaledHeight; ++i) {
            
            src = rawImageData + bufferRowBytes * i;
            
            //src = (unsigned char*)rawImageData + bufferRowBytes * i;
            dst = baseAddress + rowbytes * (scaledHeight - 1 - i);
                        
            memmove(dst, src, scaledWidth * 4);
        }
        
        
        // write the sample
        
        // NO AVAssetWriter....
        
        
        // AVCaptureSession?
        
        
        BOOL appended = [assetWriterPixelBufferAdaptor appendPixelBuffer:pixelBuffer withPresentationTime:presentationTime];
        
        // cleanup
        CVPixelBufferUnlockBaseAddress(pixelBuffer, 0);
        CVBufferRelease(pixelBuffer);
        
        //BOOL appended = YES;
        
        // check if the append operation returned true; if not, report an error
        if (appended) {
			//printf("appended sample at time %lf\n", CMTimeGetSeconds(presentationTime));
		}
        else {
			printf("failed to append\n");
			[self stopRecording];
			//self.startStopButton.selected = NO;
		}
	}
    else {
        printf("assetWriterInput was not ready for more media data for frame with value: %lld and timescale: %d\n",presentationTime.value,presentationTime.timescale);
    }
}

-(void) startRecording {
        
    // the only variables that I'm concerned about are...
    
    // - the filename (need to keep it different for each new file)
        
    // create the AVAssetWriter
	NSString *moviePath = [[self pathToDocumentsDirectory] stringByAppendingPathComponent:OUTPUT_FILE_NAME];
	if ([[NSFileManager defaultManager] fileExistsAtPath:moviePath]) {
		[[NSFileManager defaultManager] removeItemAtPath:moviePath error:nil];
	}
	
	NSURL *movieURL = [NSURL fileURLWithPath:moviePath];
	NSError *movieError = nil;

	assetWriter = [[AVAssetWriter alloc] initWithURL: movieURL 
                                            fileType: AVFileTypeQuickTimeMovie 
                                               error: &movieError];
    
    // set up an AVAssetWriterInput
    //NSDictionary *videoCompressionProps = [NSDictionary dictionaryWithObjectsAndKeys:
    //                                       [NSNumber numberWithDouble:128.0*1024.0], AVVideoAverageBitRateKey,
    //                                       nil ];
    
	NSDictionary *assetWriterInputSettings = [NSDictionary dictionaryWithObjectsAndKeys:
											  AVVideoCodecH264, AVVideoCodecKey,
											  [NSNumber numberWithInt:FRAME_WIDTH*frameScale], AVVideoWidthKey,
											  [NSNumber numberWithInt:FRAME_HEIGHT*frameScale], AVVideoHeightKey,
											  //videoCompressionProps, AVVideoCompressionPropertiesKey,
                                              nil];
	assetWriterInput = [AVAssetWriterInput assetWriterInputWithMediaType: AVMediaTypeVideo
														  outputSettings:assetWriterInputSettings];
	assetWriterInput.expectsMediaDataInRealTime = NO;
	[assetWriter addInput:assetWriterInput];
	
    // set up an AVAssetWriterPixelBufferAdaptor
    
    NSDictionary *pixelBufferAttributes = [NSDictionary dictionaryWithObjectsAndKeys:
                                           [NSNumber numberWithInt:kCVPixelFormatType_32BGRA],kCVPixelBufferPixelFormatTypeKey,
                                           [NSNumber numberWithInt:FRAME_WIDTH*frameScale],kCVPixelBufferWidthKey,
                                           [NSNumber numberWithInt:FRAME_HEIGHT*frameScale],kCVPixelBufferHeightKey,
                                           [NSNumber numberWithInt:((FRAME_WIDTH*frameScale))*4],kCVPixelBufferBytesPerRowAlignmentKey,
                                           
                                           // ^^^ assuming 3 instead of 4 because there's no alpha channel present... but that's with unsigned chars!
                                           
                                           // with an unsigned SHORT 565, it should be one short per set of rgb channels. So *2?
                                           //[NSNumber numberWithBool:YES], kCVPixelBufferCGImageCompatibilityKey,
                                           //[NSNumber numberWithBool:YES], kCVPixelBufferCGBitmapContextCompatibilityKey,
                                           //[NSNumber numberWithBool:YES], kCVPixelBufferOpenGLCompatibilityKey,
                                           nil];
    
    
	assetWriterPixelBufferAdaptor = [[AVAssetWriterInputPixelBufferAdaptor alloc]
									 initWithAssetWriterInput:assetWriterInput
									 sourcePixelBufferAttributes:pixelBufferAttributes];
    
    [assetWriter startWriting];
    
    // be sure to call this before we start writing samples... fetching the CMSampleBuffer as we've demonstrated before.

    // allocate the raw data buffer
    NSInteger dataLength = ((FRAME_WIDTH*frameScale)*(FRAME_HEIGHT*frameScale))*4; // Length of an uncompressed RGB bitmap
    rawImageData = valloc(dataLength * sizeof(GLubyte));         // Temporary; we want a CVPixelBuffer that does this instead
    
    scaledWidth = FRAME_WIDTH*frameScale;
    scaledHeight = FRAME_HEIGHT*frameScale;
    
    bgraData = malloc(scaledWidth*scaledHeight*4*sizeof(GLubyte));
    
    // precalculate the size of each row, to optimize for valloc
    bufferRowBytes = (scaledWidth * 4 + 63) & ~63;
    
    isFirstFrame = YES;
    isRecording = YES;
}

-(void) stopRecording {
    
    // check to make sure everything's okay, then wrap up all recording operations
    
	[assetWriter finishWriting];
    
    NSLog(@"finished writing to file %@",OUTPUT_FILE_NAME);

    assetWriter = nil;    
    assetWriterInput = nil;
    assetWriterPixelBufferAdaptor = nil;
    
    free(rawImageData);
    rawImageData = nil;
    
    free(bgraData);
    bgraData = nil;
    
    isRecording = NO;
    
    // consider adding functionality to call UISaveVideoAtPathToSavedPhotosAlbum(...) and related functions on a separate thread, so the user doesn't have to worry about dealing with the app's documents directory
}

@end
