//
// This file is part of AlphaCam II.
//
// AlphaCam II is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AlphaCam II is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with AlphaCam II.  If not, see <http://www.gnu.org/licenses/>.
//
//
//  NLGLCameraViewController.h
//  AlphaCam II
//
//  Created by Nicholas Levin on 5/6/11.
//  Copyright 2011 Studio Andromeda. All rights reserved.
//

#import <UIKit/UIKit.h>
#import "NLGLCamera.h"
#import "NLGLView.h"
#import "NLGLCameraVideoRecorder.h"
#import "ESRenderer.h"

typedef enum { PASSTHROUGH_VIDEO, SIMPLE_DITHERING } GLCameraDisplayMode;

@interface NLGLCameraViewController : UIViewController <NLGLCameraDelegate,ESRenderer>
{
	NLGLCamera *camera;
	UIScreen *screenForDisplay;
	NLGLView *glView;
	
    BOOL displayModeHasChanged;
	GLCameraDisplayMode displayMode;
	
	CGPoint currentTouchPoint;
	
    // shader programs
	GLuint directDisplayProgram, ditherProgram;
    
    // texture buffers
	GLuint videoFrameTexture,LUTTexture;//[3];
    GLuint pboIndex, pboNextIndex;
    
    // VBOs
    GLuint vboIds[2];
    
    // recording
    NLGLCameraVideoRecorder *videoRecorder;
}

@property(readonly) NLGLView *glView;

// Initialization and teardown
- (id)initWithScreen:(UIScreen *)newScreenForDisplay;

// OpenGL ES 2.0 setup methods
- (BOOL)loadVertexShader:(NSString *)vertexShaderName fragmentShader:(NSString *)fragmentShaderName forProgram:(GLuint *)programPointer;
- (BOOL)compileShader:(GLuint *)shader type:(GLenum)type file:(NSString *)file;
- (BOOL)linkProgram:(GLuint)prog;
- (BOOL)validateProgram:(GLuint)prog;

// Display mode switching
- (void)handleSwitchOfDisplayMode:(id)sender;

// Texture caching
- (GLuint)createTextureUsingWidth:(GLuint)w Height:(GLuint)h;

// Attribute and uniform initialization methods
- (void)initGLConstantValues;
- (UIImage*)loadLUTImageWithFilename:(NSString*)filename;
- (uint8_t*)getLUTAsBitmapFromUIImage:(UIImage*)LUTImage;
- (void)loadLUTTexture;

@end