//
// This file is part of AlphaCam II.
//
// AlphaCam II is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AlphaCam II is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with AlphaCam II.  If not, see <http://www.gnu.org/licenses/>.
//
//
//  NLMOPalette.m
//  AlphaCam II
//
//  Created by Nicholas Levin on 6/4/11.
//  Copyright (c) 2011 Studio Andromeda. All rights reserved.
//

#import "NLMOPalette.h"
#import "NLMOColor.h"
#import "NLMOOldComputers.h"


@implementation NLMOPalette
@dynamic timeStamp;
@dynamic color;
@dynamic oldComputerType;

- (void)addColorObject:(NLMOColor *)value {    
    NSSet *changedObjects = [[NSSet alloc] initWithObjects:&value count:1];
    [self willChangeValueForKey:@"color" withSetMutation:NSKeyValueUnionSetMutation usingObjects:changedObjects];
    [[self primitiveValueForKey:@"color"] addObject:value];
    [self didChangeValueForKey:@"color" withSetMutation:NSKeyValueUnionSetMutation usingObjects:changedObjects];
}

- (void)removeColorObject:(NLMOColor *)value {
    NSSet *changedObjects = [[NSSet alloc] initWithObjects:&value count:1];
    [self willChangeValueForKey:@"color" withSetMutation:NSKeyValueMinusSetMutation usingObjects:changedObjects];
    [[self primitiveValueForKey:@"color"] removeObject:value];
    [self didChangeValueForKey:@"color" withSetMutation:NSKeyValueMinusSetMutation usingObjects:changedObjects];
}

- (void)addColor:(NSSet *)value {    
    [self willChangeValueForKey:@"color" withSetMutation:NSKeyValueUnionSetMutation usingObjects:value];
    [[self primitiveValueForKey:@"color"] unionSet:value];
    [self didChangeValueForKey:@"color" withSetMutation:NSKeyValueUnionSetMutation usingObjects:value];
}

- (void)removeColor:(NSSet *)value {
    [self willChangeValueForKey:@"color" withSetMutation:NSKeyValueMinusSetMutation usingObjects:value];
    [[self primitiveValueForKey:@"color"] minusSet:value];
    [self didChangeValueForKey:@"color" withSetMutation:NSKeyValueMinusSetMutation usingObjects:value];
}

- (void)createColorWithRed:(int)redValue green:(int)greenValue andBlue:(int)blueValue inEntity:(NSEntityDescription*)colorEntity andContext:(NSManagedObjectContext*)context
{
    NLMOColor *newColor = [NSEntityDescription insertNewObjectForEntityForName:[colorEntity name] inManagedObjectContext:context];
    
    newColor.red = [NSNumber numberWithInt:redValue];
    newColor.green = [NSNumber numberWithInt:greenValue];
    newColor.blue = [NSNumber numberWithInt:blueValue];
    
    [self addColorObject:newColor];
}

@end
