//
// This file is part of AlphaCam II.
//
// AlphaCam II is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AlphaCam II is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with AlphaCam II.  If not, see <http://www.gnu.org/licenses/>.
//
//
//  NLNearestColorLUTFactory.h
//  AlphaCam II
//
//  Created by Nicholas Levin on 5/22/11.
//  Copyright 2011 Studio Andromeda. All rights reserved.
//

#import "NLKDTree.hh"

class NLNearestColorLUTFactory {
    
protected:
    
    RGBPalette imagePalette;
    
    RGBTriple nearestColorLUT[32][32][32];
    bool createdLUT;
    
    KDTree <unsigned char> *paletteTree;    // <- optional, for more complex data sets
    
    const unsigned char findNearestColor(const uint8_t &red, const uint8_t &green, const uint8_t &blue) ;

    void applyNearestColor(uint8_t &red,  uint8_t &green,  uint8_t &blue) ;
    
public:
    
    NLNearestColorLUTFactory(RGBPalette &inputPalette,bool useKDTree = false);
    ~NLNearestColorLUTFactory();
    
    void setInputPalette(RGBPalette &inputPalette);
    void activateKDTree(bool useTree);
    
    void createLUT();    
    void writeLUTToPNG(const char* filename);
};