//
// This file is part of AlphaCam II.
//
// AlphaCam II is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// AlphaCam II is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with AlphaCam II.  If not, see <http://www.gnu.org/licenses/>.
//
//
//  NLPaletteEditorController.m
//  AlphaCam II
//
//  Created by Nicholas Levin on 5/26/11.
//  Copyright 2011 Studio Andromeda. All rights reserved.
//

#import "NLPaletteEditorController.h"
#import "NLCoreDataController.h"
#import "NLMOIIgs.h"
#import "NLMOMacClassic.h"
#import "NLMOPCjr.h"
#import "NLMOC64.h"
#import "NLMOGameBoy.h"
#import "NLMOPalette.h"
#import "NLMOColor.h"
#import "NLColorsEditorController.h"
#import "NLNearestColorLUTFactory.hh"
#import "AlphaCamRootViewController.h"

@implementation NLPaletteEditorController

@synthesize navigationController, coreDataController, fetchedResultsController=__fetchedResultsController, rootViewController; //oldComputerPalettes;

- (id)initWithStyle:(UITableViewStyle)style oldComputerRowNumber:(NSUInteger)oldComputerId coreDataController:(NLCoreDataController*)CDController
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
                
        self.coreDataController = CDController;
        
        // TODO: Wrap all of these cases into a single enumerated constant!
        switch (oldComputerId) {
            case 0:  // Apple IIgs
                self.title = @"Apple IIgs";
                currentMachine = @"NLMOIIgs";
                                
                //printf("Found a matching case, given an oldComputerId of %i\n",oldComputerId);
                break;
            
            case 1:  // Mac Classic
                self.title = @"Mac Classic";
                currentMachine = @"NLMOMacClassic";
                
                //printf("Found a matching case, given an oldComputerId of %i\n",oldComputerId);
                break;
            
            case 2:  // PCjr
                self.title = @"IBM PCjr";
                currentMachine = @"NLMOPCjr";
                                
                //printf("Found a matching case, given an oldComputerId of %i\n",oldComputerId);
                break;
            
            case 3:  // Commodore 64
                self.title = @"Commodore 64";
                currentMachine = @"NLMOC64";
                
                //printf("Found a matching case, given an oldComputerId of %i\n",oldComputerId);
                break;
            
            case 4:  // Game Boy
                self.title = @"Game Boy";
                currentMachine = @"NLMOGameBoy";
                
                //printf("Found a matching case, given an oldComputerId of %i\n",oldComputerId);
                break;
            
            default:
                printf("ERROR: Count not find a matching case, given an oldComputerId of %i\n",oldComputerId);
                                
                break;
        }
        
        MOmachineID = oldComputerId;
        
        UIBarButtonItem *addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(createNewPalette)];
        self.navigationItem.rightBarButtonItem = addButton;
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{    
    //cell.textLabel.text = [[managedObject valueForKey:@"timeStamp"] description];
    cell.textLabel.text = @"Palette";

    cell.accessoryType = UITableViewCellAccessoryDetailDisclosureButton;
}

-(void) createNewPalette
{    
    NSManagedObjectContext *context = [self.fetchedResultsController managedObjectContext];
    
    // create with default palette    
    id thisMachine = [NSEntityDescription insertNewObjectForEntityForName:currentMachine inManagedObjectContext:context];

    [thisMachine createWithDefaultPaletteInContext:context];
        
    // save the context
    [self.coreDataController saveContext];
}

-(void) deletePalette:(NSManagedObject *)palette
{
    NSManagedObjectContext *context = [self.fetchedResultsController managedObjectContext];
    
    [context deleteObject:palette];
    [self.coreDataController saveContext];
}


#pragma mark - View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return [[self.fetchedResultsController sections] count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    id <NSFetchedResultsSectionInfo> sectionInfo = [[self.fetchedResultsController sections] objectAtIndex:section];
    return [sectionInfo numberOfObjects];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"OldComputerPaletteCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    }
    
    // Configure the cell...
    [self configureCell:cell atIndexPath:indexPath];

    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete)
    {
        [self deletePalette:[self.fetchedResultsController objectAtIndexPath:indexPath]];
        [self.coreDataController saveContext];
    }   
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // The table view should not be re-orderable.
    return NO;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{            
    NLMOOldComputers *selectedComputer = [self.fetchedResultsController objectAtIndexPath:indexPath];
    
    RGBPalette selectedPalette;
    
    NLMOPalette* managedObjectPalette = selectedComputer.palette;
    
    NSSet* MOPaletteColorSet = managedObjectPalette.color;
    
    selectedPalette.size = [MOPaletteColorSet count];
	
    selectedPalette.table = static_cast<RGBTriple *>( malloc(sizeof(RGBTriple) * selectedPalette.size));
	
    uint channelId = 0;
    
    for (NLMOColor* color in MOPaletteColorSet) {
        selectedPalette.table[channelId].channel[0] = [color.red intValue];
        selectedPalette.table[channelId].channel[1] = [color.green intValue];
        selectedPalette.table[channelId].channel[2] = [color.blue intValue];
        
        ++channelId;
    }
    
    NLNearestColorLUTFactory paletteLUT = NLNearestColorLUTFactory(selectedPalette);
    paletteLUT.createLUT();
    paletteLUT.writeLUTToPNG("LUTOutput.png");
    
    // TODO: Dismiss this entire sequence of Modally presented View Controllers to bring the user back to the Root controller WITHOUT calling the application delegate.
    
    [rootViewController dismissModalViewControllerAnimated:YES];
}

// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return YES if you want the specified item to be editable.
    return YES;
}

#pragma mark - Fetched results controller

- (NSFetchedResultsController *)fetchedResultsController
{
    if (__fetchedResultsController != nil)   // seems to be going in an infinite loop here. Hunh.
    {
        return __fetchedResultsController;
    }
    
    /*
     Set up the fetched results controller.
     */
    // Create the fetch request for the entity.
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    // Edit the entity name as appropriate.
    NSEntityDescription *entity = [NSEntityDescription entityForName:currentMachine inManagedObjectContext:coreDataController.managedObjectContext];
    [fetchRequest setEntity:entity];
    
    // Set the batch size to a suitable number.
    [fetchRequest setFetchBatchSize:20];
    
    // Edit the sort key as appropriate.
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"timeStamp" ascending:NO];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
    
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    // Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
    NSFetchedResultsController *aFetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:coreDataController.managedObjectContext sectionNameKeyPath:nil cacheName:@"Root"];
    aFetchedResultsController.delegate = self;
    self.fetchedResultsController = aFetchedResultsController;
    
    
	NSError *error = nil;
	if (![self.fetchedResultsController performFetch:&error])   // eventually crashes over here. Strange... why is it calling this method more than once?
    {
	    /*
	     Replace this implementation with code to handle the error appropriately.
         
	     abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
	     */
	    NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
	    abort();
	}
    
    return __fetchedResultsController;
}    

#pragma mark - Method for detail disclosure button at row

- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath 
{
    // fetching an NLMOOldComputers subclass (such as NLMOIIgs)
    NLMOOldComputers *selectedComputer = [self.fetchedResultsController objectAtIndexPath:indexPath];
    
    NLColorsEditorController *colorsEditorViewController = [[NLColorsEditorController alloc] initWithStyle:UITableViewStylePlain coreDataController:self.coreDataController andPalette:selectedComputer.palette];
        
    [self.navigationController pushViewController:colorsEditorViewController animated:YES];
    
}

#pragma mark - Fetched results controller delegate

- (void)controllerWillChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView beginUpdates];
}

- (void)controller:(NSFetchedResultsController *)controller didChangeSection:(id <NSFetchedResultsSectionInfo>)sectionInfo
           atIndex:(NSUInteger)sectionIndex forChangeType:(NSFetchedResultsChangeType)type
{
    switch(type)
    {
        case NSFetchedResultsChangeInsert:
            [self.tableView insertSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [self.tableView deleteSections:[NSIndexSet indexSetWithIndex:sectionIndex] withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controller:(NSFetchedResultsController *)controller didChangeObject:(id)anObject
       atIndexPath:(NSIndexPath *)indexPath forChangeType:(NSFetchedResultsChangeType)type
      newIndexPath:(NSIndexPath *)newIndexPath
{
    UITableView *tableView = self.tableView;
    
    switch(type)
    {
            
        case NSFetchedResultsChangeInsert:
            [tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeDelete:
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
            break;
            
        case NSFetchedResultsChangeUpdate:
            [self configureCell:[tableView cellForRowAtIndexPath:indexPath] atIndexPath:indexPath];
            break;
            
        case NSFetchedResultsChangeMove:
            [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:newIndexPath]withRowAnimation:UITableViewRowAnimationFade];
            break;
    }
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
{
    [self.tableView endUpdates];
}

/*
 // Implementing the above methods to update the table view in response to individual changes may have performance implications if a large number of changes are made simultaneously. If this proves to be an issue, you can instead just implement controllerDidChangeContent: which notifies the delegate that all section and object changes have been processed. 
 
 - (void)controllerDidChangeContent:(NSFetchedResultsController *)controller
 {
 // In the simplest, most efficient, case, reload the table view.
 [self.tableView reloadData];
 }
 */


@end
